﻿/**
 * Systemwalker Centric Manager AR連携サンプル
 * Copyright 2015 FUJITSU LIMITED
 */

/**
 * 業務アプリサンプルのライブラリ空間です。
 */
var Apl = {};

/** 使用するシナリオIDとシナリオID、シーンID、マーカIDの最大値です。 */
Apl.useScenarioId = Property.getUseScenarioId();

/**
 * 動作モードを保持します。
 * Webストレージ保存対象です。
 */
Apl.operationMode;

/**
 * シナリオリストを保持します。
 * Webストレージ保存対象です。
 */
Apl.scenarioList = null;

/**
 * シナリオIDを保持します。
 * Webストレージ保存対象です。
 */
Apl.scenarioId = null;

/** マーカー検知に登録したリスナーIDです。*/
Apl.listenerId = "";

Apl.operationModeSuccessError=false;

/** AR実行サーバのquad検索のクエリを表現するクラスです。 */
Apl.QuadQuery = function(){
	this.type;
	this.limitRange;
	this.qattributeOrderIndexRange;
	this.qtypeNameRages;
	this.whereExpressions;
	this.sortOrderExpressions;
	return this;
};

/**
 * AR実行サーバのquad検索クエリの範囲を表現するクラスです。
 * コンストラクタでendを省略するとstartと同じ値が格納されます。
 */
Apl.Range = function(_start, _end){
	this.start = _start;
	if(typeof _end == "undefined") this.end = _start;
	else this.end = _end;
	return this;
};

/** AR実行サーバのquad検索クエリのExpressionを表現するクラスです。 */
Apl.QuadQueryExpression = function(_nameRange, _valueRange, _type, _desc){
	this.qattributeNameRanges = _nameRange;
	this.qvalueRanges = _valueRange;
	this.qvalueType = _type;
	this.desc = _desc;
	return this;
};

/** QuadQueryクラスからquad検索文字列を生成します。 */
Apl.makeQuadQuery = function(_qQuery){
	var query="quads?";
	if(_qQuery == null) return query;
	if(_qQuery.type != null) query += "type=" + _qQuery.type;
	if(_qQuery.limitRange != null) query += "&limitRange=" + JSON.stringify(_qQuery.limitRange);
	if(_qQuery.qattributeOrderIndexRange != null) query += "&qattributeOrderIndexRange=" + JSON.stringify(_qQuery.qattributeOrderIndexRange);
	if(_qQuery.qtypeNameRanges != null) query += "&qtypeNameRanges=" + JSON.stringify(_qQuery.qtypeNameRanges);
	if(_qQuery.whereExpressions != null) query += "&whereExpressions=" + JSON.stringify(_qQuery.whereExpressions);
	if(_qQuery.sortOrderExpressions != null) query += "&sortOrderExpressions=" + JSON.stringify(_qQuery.sortOrderExpressions);
	return query;
};

/** 何もしないメソッドです。 */
Apl.noop = function(_result){};

/** WebStorageに保存した全てのデータを取得します。 */
Apl.getLocalStorageData = function(){

	//動作モードの取得
	if(localStorage.getItem("operationMode") != null)
		Apl.operationMode = localStorage.getItem("operationMode");
	//シナリオリストの取得
	if(localStorage.getItem("scenarioList") != null)
		Apl.scenarioList = JSON.parse(localStorage.getItem("scenarioList"));
	//現在のシナリオIDの取得
	if(localStorage.getItem("scenarioId") != null)
		Apl.scenarioId = parseInt(localStorage.getItem("scenarioId"));
};

/** エラーログを出力します。 */
Apl.log = function(_message, _detail){
	if(_message == null) return;
	var code = 0;
	var message;

	if(_message instanceof Error){ //Errorオブジェクトの場合
		if(typeof _message.code == 'number') code = _message.code;
		message = _message.componentName == null ? _message.toString() : _message.componentName + " : " + _message.toString();
		if(_message.cause != null) message += " Cause : "+_message.cause;
	} else message = _message.toString(); //それ以外の場合
	//messageはアラートで表示します。
	alert(message);
	//message + detailをログに出力します。
	if(_detail) message +="\n" + _detail;
	try{
		AR.Log.log(AR.Log.LevelType.ERROR,code,message,Apl.noop, Apl.logError);
	} catch(e){
		alert("ログの出力に失敗しました。"+e);
	}
};

/** ログ出力のonErrorに設定するコールバック関数です。 */
Apl.logError = function(_result){
	alert("ログの出力に失敗しました。\n" + result.getStatus() + "\n"+ _result.getValue());
};

/**
 * ネイティブカメラ起動のonErrorに設定するコールバック関数です。
 */
Apl.startCameraViewError = function(_result){
	var message = "ネイティブカメラの起動に失敗しました。";
	var detail = _result.getStatus() + "\n"+ _result.getValue();
	Apl.log(message, detail);
};

/** ネイティブカメラ停止のonErrorに設定するコールバック関数です。 */
Apl.stopCameraViewError = function(_result){
	var message = "ネイティブカメラの停止に失敗しました。";
	var detail = _result.getStatus() + "\n"+ _result.getValue();
	Apl.log(message, detail);
};

/** マーカー検知登録のonErrorに設定するコールバック関数です。*/
Apl.addMarkerListenerError = function(_result){
	var message ="マーカー検知のイベントリスナ登録に失敗しました。";
	var detail = _result.getStatus() + "\n"+ _result.getValue();
	Apl.log(message, detail);
};

/** マーカー検知削除のonErrorに設定するコールバック関数です。 */
Apl.removeMarkerListenerError = function(_result){
	var message = "マーカー検知のイベントリスナ削除に失敗しました。\n";
	var detail = _result.getStatus() + "\n"+ _result.getValue();
	Apl.log(message, detail);
};

/** オフラインストレージ設定のonErrorに設定するコールバック関数です。 */
Apl.useOfflineStorageError = function(_result){
	var message = "オフラインストレージの設定に失敗しました。";
	var detail = _result.getStatus() + "\n"+ _result.getValue();
	Apl.log(message, detail);
};

/**
 * AR実行サーバからデータ取得のonErrorに設定するコールバック関数です。
 * ネイティブでHTTP例外が発生した場合は結果オブジェクトのvalueの形式が異なります。
 */
Apl.getArServerDataError = function(_result){
	var message = AR_workMan+"\nAR実行サーバのデータ取得に失敗しました。動作モードとネットワーク状況を確認して再度お試しください。";
	var detail = null;
	if(_result.getStatus() == "AR_HTTP_EXCEPTION"){
		detail = _result.getValue().status + " : " + _result.getValue().statusText + "\n" + _result.getValue().responseText;
	} else {
		detail += _result.getStatus() + "\n"+ _result.getValue();
	}
	Apl.log(message, detail);
};

/** 動作モード取得のonErrorに設定するコールバック関数です。 */
Apl.getOperationModeError = function(_result){
	if(Apl.operationModeSuccessError) {
		alert("サーバ通信モードで実行してください。");
	} else {
		var message = "動作モードの取得に失敗しました。\n";
		var detail = _result.getStatus() + "\n"+ _result.getValue();
		Apl.log(message, detail);
	}
};

/** タップイベント通知のonErrorに設定するコールバック関数です。 */
Apl.onBodyClickError = function(_result){
	var message = "タップイベントの通知に失敗しました。\n";
	var detail = _result.getStatus() + "\n"+ _result.getValue();
	Apl.log(message, detail);
};

/** AR重畳表示定義データ追加のonErrorに設定するコールバック関数です。 */
Apl.putError = function(_result){
	var message = "AR重畳表示定義データの追加に失敗しました。\n";
	var detail = _result.getStatus() + "\n"+ _result.getValue();
	Apl.log(message, detail);
};

/** ARコンテンツ削除のonErrorに設定するコールバック関数です。 */
Apl.removeError = function(_result){
	var message = "ARコンテンツの削除に失敗しました。\n";
	var detail = _result.getStatus() + "\n"+ _result.getValue();
	Apl.log(message, detail);
};

/** ARコンテンツ全削除のonErrorに設定するコールバック関数です。 */
Apl.clearError = function(_result){
	var message = "ARコンテンツの全削除に失敗しました。\n";
	var detail = _result.getStatus() + "\n"+ _result.getValue();
	Apl.log(message, detail);
};

/**
 * index.htmlの画面初期化処理です。
 * ネイティブカメラの停止、Webストレージから保存したデータを取得した後、動作モードを判定します。
 */
Apl.onloadIndex = function(){
	// ネイティブカメラを停止させます。
	try{
		AR.Camera.stopCameraView(Apl.noop, Apl.stopCameraViewError);
	} catch (e){
		Apl.log(e);
	}

	//オフラインストレージをONに設定します。
	try{
		AR.Data.useOfflineStorage(true, Apl.noop,Apl.useOfflineStorageError);
	} catch (e){
		Apl.log(e);
	}

	//Webストレージに保存した全てのデータを取得します。
	Apl.getLocalStorageData();

	Apl.scenarioId = Apl.useScenarioId;

	//動作モードを取得してApl.operationModeに保存します。
	try{
		AR.OS.getOperationMode(Apl.getOperationModeSuccess ,Apl.getOperationModeError);
	} catch (e){
		Apl.log(e);
	}
};

/** 取得した動作モードを変数に格納します。 */
Apl.getOperationModeSuccess = function(_result){

	//取得した動作モードを変数に格納します。
	Apl.operationMode = _result.getValue();
	localStorage.setItem("operationMode", Apl.operationMode);

	//動作モードに応じてhtml表示を変更します。
	Apl.showOperationMode();

	if(Apl.operationMode == "serverMode"){ //サーバ通信モードの場合
		//AR実行サーバからシナリオリストを取得します。
		Apl.getScenario();
	} else { //スタンドアローンモードの場合
		Apl.operationModeSuccessError=true;
		throw new Error("サーバ通信モードで実行してください。");
	}
};

/**
 * 動作モードに応じてhtmlの表示を変更します。
 */
Apl.showOperationMode = function(){
	if(Apl.operationMode == "serverMode"){
		document.getElementById("operationMode_area").innerText = "サーバ通信モード";
		document.getElementById("operationMode_area").className = "serverMode";
	} else {
		Apl.operationModeSuccessError=true;
		throw new Error("サーバ通信モードで実行してください。");
	}
};

//******************************************************************************
//*	Apl.startScenario:	シナリオを開始する。
//******************************************************************************
Apl.startScenario = function(){
	try{
		if(Apl.useScenarioId != null){
			if(Apl.scenarioList[Apl.useScenarioId] != null) {
				//現在のシナリオIDをWebストレージへ保存します。
				Apl.workPage = "work.html";
				//現在のシナリオIDを退避します
				localStorage.setItem("scenarioId", Apl.useScenarioId);
				//作業用ページのURLを退避します
				localStorage.setItem("workPage", Apl.workPage);
				Apl.start();
			} else {
				alert("シナリオを取得できません。設定したシナリオIDでシナリオが登録されていることや動作モードとネットワーク状態を確認してもう一度お試しください。");
			}
		} else {
			alert("シナリオを取得できません。設定したシナリオIDでシナリオが登録されていることや動作モードとネットワーク状態を確認してもう一度お試しください。");
		}
	}catch(e){
		alert("シナリオを取得できません。設定したシナリオIDでシナリオが登録されていることや動作モードとネットワーク状態を確認してもう一度お試しください。");
	}
};

//******************************************************************************
//*	Apl.getScenario:	AR実行サーバからシナリオデータを取得する
//******************************************************************************
/** シナリオ取得の検索クエリを生成し、AR実行サーバからシナリオデータを取得します。 */
Apl.getScenario = function(){
	//quads検索用クエリオブジェクトを作成します。
	var query = new Apl.QuadQuery();
	query.type = "RECORDSANDCOUNT";
	query.limitRange = new Apl.Range(1,10);
	query.qattributeOrderIndexRange = new Apl.Range(1,10);
	//シナリオが登録されているqtypeを指定します。
	query.qtypeNameRanges = new Apl.Range("arscn_scenario");
	//シナリオIDを指定します。
	query.whereExpressions = new Apl.QuadQueryExpression(new Apl.Range("ar_id"), new Apl.Range(Apl.useScenarioId), "LONG");
	//文字列に変換します。
	query = Apl.makeQuadQuery(query);
	//AR実行サーバからシナリオリストを取得します。
	try{
		AR.Data.getArServerData(query, true, Apl.getScenarioSuccess, Apl.getArServerDataError);
	} catch(e){
		Apl.log(e);
	}
};

/** AR実行サーバからシナリオ取得に成功した場合に呼ばれます。 */
Apl.getScenarioSuccess = function(_result){

	//初期化します
	Apl.scenarioList = null;
	//結果からシナリオを抽出してApl.scenarioListに格納します。
	Apl.extractScenario(_result.getValue());

	//シナリオIDをWebストレージに保存します。
	localStorage.setItem("scenarioId", Apl.scenarioId);
	//シナリオが登録されていた場合Webストレージに保存します。
	if(Apl.scenarioList != null)
		localStorage.setItem("scenarioList", JSON.stringify(Apl.scenarioList));

	//取得したシナリオを開始します。
	Apl.startScenario();
};

/** JSONオブジェクトからシナリオを抽出します。 */
Apl.extractScenario = function(_data){
	//取得したレコード数です。
	var recordLength = _data.records.length;
	for(var recordIndex = 0; recordIndex < recordLength; recordIndex++){
		//レコードを一つずつ調べます。
		var record = _data.records[recordIndex];
		var valueLength = record.qvalues.length;
		var scenarioId = null;
		var value = new Object();

		//使用するqvalueの値だけ取得します。attributeNameで判断します。
		for(var valueIndex = 0; valueIndex < valueLength; valueIndex++){
			var qvalue = record.qvalues[valueIndex];
			if(qvalue.qattributeName == "ar_name"){ // シナリオ名
				value.name = qvalue.stringValue;
			}
			if(qvalue.qattributeName == "ar_description"){ // シナリオの説明
				value.description = qvalue.stringValue;
			}
			if(qvalue.qattributeName == "ar_id"){ // シナリオID
				scenarioId = qvalue.longValue;
			}
		}

		if(value.name != null && scenarioId != null){
			//Apl.scenarioListに格納します。
			if(Apl.scenarioList == null) Apl.scenarioList = new Array();
			Apl.scenarioList[scenarioId] = value;
		}
	}
};

/** シナリオを開始した時に呼ばれます。*/
Apl.start = function(){
	//ネイティブAR表示層に設定したAR重畳表示定義データを全てクリアします。
	try{
		AR.Renderer.clear(Apl.clearSuccess,Apl.clearError);
	} catch(e){
		Apl.log(e);
	}
};

/** AR重畳表示定義データのクリアに成功した場合 */
Apl.clearSuccess = function(_result){
	//作業用ページのURLに遷移します。
	location.href = Apl.workPage;
};

/**
 * work.htmlの画面初期化処理です。
 * Webストレージに保存したデータの取得、画面表示の変更、カメラの起動、マーカー検知イベントリスナを登録します。
 */
Apl.onloadWork = function() {
	//Webストレージに保存した全てのデータを取得します。
	Apl.getLocalStorageData();

	//動作モードに応じてhtml表示を変更します。
	Apl.showOperationMode();

	//画面タップのイベントリスナを登録します。
	if(window.navigator.userAgent.match(/(iPad|iPhone|iPod)/i))
		document.addEventListener("touchstart", Apl.clickEvent, false);
	else document.addEventListener("click", Apl.clickEvent, false);

	//ネイティブカメラを起動させます。
	try{
		AR.Camera.startCameraView(Apl.noop, Apl.startCameraViewError);
	} catch (e){
		Apl.log(e);
	}
	//マーカー検知のイベントリスナ追加です。
	try{
		AR.Camera.addMarkerListener( Apl.addMarkerListenerSuccess, Apl.addMarkerListenerError, Apl.onDetectMarker, Apl.noop);
	}catch(e){
		Apl.log(e);
	}
};

/**
 * マーカー検知登録のonSuccessに設定するコールバック関数です。
 */
Apl.addMarkerListenerSuccess = function(_result){
	//リスナIDをApl.listenerIdに格納します。登録したマーカー検知イベントリスナの削除に使用します。
	Apl.listenerId = _result.getValue();
};

//******************************************************************************
//*	Apl.onDetectMarker:	ネイティブカメラがマーカを検知した時の処理を行うメソッド
//*						マーカー検知用の成功メソッドです。
//*						マーカーを検知した場合に呼び出されます。
//******************************************************************************
Apl.onDetectMarker = function(_result){
	try{
		// 引数に検知したマーカ情報が含まれています。
		var markval = _result.getValue();
		if(markval.status == true){	 //検出した場合
			//マーカー検知通知領域の表示を変更します。
			document.getElementById("detect").innerHTML = "マーカー"+markval.markerId+"を検出しました。View="+AR_cameraView;
			if(AR_cameraView == true && markval.markerId != AR_markerId){
				AR_markerId = markval.markerId;
				getServerStatus(AR_markerId);
			}
		}
	}catch(e){
		Apl.log(e);
	}
};

/** クリックイベントが発生したときに呼ばれます。 */
Apl.clickEvent = function(){
	//ネイティブアプリケーションにクリック座標を通知します。
	try{
		if(event)
			AR.OS.onBodyClick(event, Apl.noop, Apl.onBodyClickError);
	} catch(e){
		Apl.log(e);
	}
};

/**
 * work.htmlの画面終了処理です。
 * マーカー検知時のイベントリスナを削除します。
 */
Apl.onUnloadWork = function(){
	//イベントリスナを削除します。
	try{
		AR.Camera.removeMarkerListener(Apl.listenerId, Apl.removeListenerId, Apl.removeMarkerListenerError);
	} catch (e){
		Apl.log(e);
	}

};

/** マーカー検知時のイベントリスナ削除に成功した場合のコールバック関数です。 */
Apl.removeListenerId = function(_result){
	Apl.listenerId = "";
};

//******************************************************************************
//*	Apl.cameraStart:	ネイティブカメラを起動します。
//******************************************************************************
Apl.cameraStart = function(){
	//ネイティブカメラを起動させます。
	try{
		AR.Camera.startCameraView(Apl.noop, Apl.startCameraViewError);
	} catch (e){
		Apl.log(e);
	}
};

//******************************************************************************
//*	Apl.cameraStop:	ネイティブカメラを停止します。
//******************************************************************************
Apl.cameraStop = function(){
	//ネイティブカメラを停止させます。
	try{
		AR.Camera.stopCameraView(Apl.noop, Apl.stopCameraViewError);
	} catch (e){
		Apl.log(e);
	}
};
